"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var postcss_1 = __importStar(require("postcss"));
var rtlcss_1 = __importDefault(require("rtlcss"));
var _types_1 = require("../@types");
var _constants_1 = require("../constants");
var selectors_1 = require("../utilities/selectors");
var containers_1 = require("../utilities/containers");
exports.parseDeclarations = function (rules, keyFrameData, rule, options) {
    var mode = options.mode, ltrPrefix = options.ltrPrefix, rtlPrefix = options.rtlPrefix, source = options.source, processUrls = options.processUrls, useCalc = options.useCalc, stringMap = options.stringMap;
    var deleteDeclarations = [];
    var ruleFlipped = rule.clone();
    var ruleFlippedSecond = rule.clone();
    ruleFlipped.removeAll();
    ruleFlippedSecond.removeAll();
    containers_1.walkContainer(rule, [_constants_1.DECLARATION_TYPE], true, function (node) {
        var decl = node;
        var declString = decl.toString() + ";";
        var declFlippedString = rtlcss_1.default.process(declString, { processUrls: processUrls, useCalc: useCalc, stringMap: stringMap });
        var root = postcss_1.default.parse(declFlippedString);
        var declFlipped = root.first;
        declFlipped.raws = decl.raws;
        var declProp = decl.prop.trim();
        var declPropUnprefixed = postcss_1.vendor.unprefixed(declProp);
        var declValue = decl.value.trim();
        var isAnimation = declPropUnprefixed === _constants_1.ANIMATION_PROP || declPropUnprefixed === _constants_1.ANIMATION_NAME_PROP;
        if (declProp === declFlipped.prop.trim() &&
            declValue === declFlipped.value.trim() &&
            (!isAnimation ||
                (isAnimation &&
                    (keyFrameData.length === 0 ||
                        !keyFrameData.regExp.test(declValue))))) {
            return;
        }
        if (isAnimation) {
            var animationNames = decl.value.match(keyFrameData.regExp);
            var animationName = animationNames[0].trim();
            var declValue_1 = decl.value.replace(animationName, keyFrameData.stringMap[animationName].name);
            var declValueFlipped = decl.value.replace(animationName, keyFrameData.stringMap[animationName].nameFlipped);
            var declCloneFlipped = decl.clone();
            if (mode === _types_1.Mode.combined) {
                var declClone = decl.clone();
                declClone.value = declValue_1;
                declCloneFlipped.value = declValueFlipped;
                ruleFlipped.append(declClone);
                ruleFlippedSecond.append(declCloneFlipped);
                deleteDeclarations.push(decl);
            }
            else {
                var declCloneFlipped_1 = decl.clone();
                decl.value = declValue_1;
                declCloneFlipped_1.value = declValueFlipped;
                ruleFlipped.append(declCloneFlipped_1);
            }
        }
        else {
            if (mode === _types_1.Mode.combined) {
                var declClone = decl.clone();
                ruleFlipped.append(declClone);
                ruleFlippedSecond.append(declFlipped);
                deleteDeclarations.push(decl);
            }
            else {
                if (_constants_1.FLIP_PROPERTY_REGEXP.test(decl.prop)) {
                    var declClone = decl.clone();
                    declClone.value = 'unset';
                    ruleFlipped.append(declClone);
                }
                ruleFlipped.append(declFlipped);
            }
        }
    });
    if (deleteDeclarations.length) {
        deleteDeclarations.forEach(function (decl) { return decl.remove(); });
    }
    if (ruleFlipped.nodes.length || ruleFlippedSecond.nodes.length) {
        if (mode === _types_1.Mode.combined) {
            selectors_1.addSelectorPrefixes(ruleFlipped, source === _types_1.Source.ltr ? ltrPrefix : rtlPrefix);
            selectors_1.addSelectorPrefixes(ruleFlippedSecond, source === _types_1.Source.rtl ? ltrPrefix : rtlPrefix);
        }
        else {
            selectors_1.addSelectorPrefixes(ruleFlipped, source === _types_1.Source.ltr ? rtlPrefix : ltrPrefix);
            selectors_1.addSelectorPrefixes(ruleFlippedSecond, source === _types_1.Source.rtl ? rtlPrefix : ltrPrefix);
        }
        rules.push({
            rule: rule,
            ruleLTR: ruleFlipped,
            ruleRTL: ruleFlippedSecond
        });
    }
};
