"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var _types_1 = require("../@types");
var _constants_1 = require("../constants");
var getStringMapName = function (map) {
    var search = Array.isArray(map.search) ? map.search[0] : map.search;
    var replace = Array.isArray(map.replace) ? map.replace[0] : map.replace;
    var reg = /[^\w]/g;
    var searchName = search.replace(reg, '-');
    var replaceName = replace.replace(reg, '_');
    return searchName + "-" + replaceName;
};
var getRTLCSSStringMap = function (stringMap) {
    return stringMap.map(function (map) { return ({
        name: getStringMapName(map),
        priority: 100,
        search: map.search,
        replace: map.replace,
        options: {
            scope: '*',
            ignoreCase: false
        }
    }); });
};
var ModeValuesArray = Object.keys(_types_1.Mode).map(function (prop) { return _types_1.Mode[prop]; });
var SourceValuesArray = Object.keys(_types_1.Source).map(function (prop) { return _types_1.Source[prop]; });
var isNotAcceptedPrefix = function (prefix) {
    if (typeof prefix !== 'string' && !Array.isArray(prefix)) {
        return true;
    }
    if (Array.isArray(prefix)) {
        return prefix.some(function (item) { return typeof item !== 'string'; });
    }
    return false;
};
var isNotAcceptedStringMap = function (stringMap) {
    if (!Array.isArray(stringMap)) {
        return true;
    }
    return stringMap.some(function (map) {
        return isNotAcceptedPrefix(map.search) || isNotAcceptedPrefix(map.replace);
    });
};
var defaultStringMap = [
    {
        search: ['left', 'Left', 'LEFT'],
        replace: ['right', 'Right', 'RIGHT']
    },
    {
        search: ['ltr', 'Ltr', 'LTR'],
        replace: ['rtl', 'Rtl', 'RTL'],
    }
];
var defaultOptions = {
    mode: _types_1.Mode.combined,
    ltrPrefix: '[dir="ltr"]',
    rtlPrefix: '[dir="rtl"]',
    source: _types_1.Source.ltr,
    processUrls: false,
    processKeyFrames: false,
    useCalc: false,
    stringMap: getRTLCSSStringMap(defaultStringMap)
};
exports.normalizeOptions = function (options) {
    var returnOptions = __assign({}, defaultOptions);
    if (options.mode && ModeValuesArray.includes(options.mode)) {
        returnOptions.mode = options.mode;
    }
    if (options.source && SourceValuesArray.includes(options.source)) {
        returnOptions.source = options.source;
    }
    if (!isNotAcceptedPrefix(options.ltrPrefix)) {
        returnOptions.ltrPrefix = options.ltrPrefix;
    }
    if (!isNotAcceptedPrefix(options.rtlPrefix)) {
        returnOptions.rtlPrefix = options.rtlPrefix;
    }
    if (typeof options.processUrls === _constants_1.BOOLEAN_TYPE) {
        returnOptions.processUrls = options.processUrls;
    }
    if (typeof options.processKeyFrames === _constants_1.BOOLEAN_TYPE) {
        returnOptions.processKeyFrames = options.processKeyFrames;
    }
    if (typeof options.useCalc === _constants_1.BOOLEAN_TYPE) {
        returnOptions.useCalc = options.useCalc;
    }
    if (!isNotAcceptedStringMap(options.stringMap)) {
        returnOptions.stringMap = getRTLCSSStringMap(options.stringMap);
    }
    return returnOptions;
};
